import { Module } from '@nestjs/common';
import { CreateAdGrpcController } from './interface/grpc-controllers/create-ad.grpc.controller';
import { CqrsModule } from '@nestjs/cqrs';
import {
  AD_REPOSITORY,
  PARAMS_PROVIDER,
  TIMEZONE_FINDER,
} from './ad.di-tokens';
import {
  MESSAGE_BROKER_PUBLISHER,
  MESSAGE_PUBLISHER,
} from '@src/app.constants';
import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';
import { AdRepository } from './infrastructure/ad.repository';
import { DefaultParamsProvider } from './infrastructure/default-params-provider';
import { MessagePublisher } from './infrastructure/message-publisher';
import { AdMapper } from './ad.mapper';
import { CreateAdService } from './core/commands/create-ad/create-ad.service';
import { TimezoneFinder } from './infrastructure/timezone-finder';
import { PrismaService } from '@libs/db/prisma.service';

@Module({
  imports: [CqrsModule],
  controllers: [CreateAdGrpcController],
  providers: [
    CreateAdService,
    PrismaService,
    AdMapper,
    {
      provide: AD_REPOSITORY,
      useClass: AdRepository,
    },
    {
      provide: PARAMS_PROVIDER,
      useClass: DefaultParamsProvider,
    },
    {
      provide: MESSAGE_BROKER_PUBLISHER,
      useClass: MessageBrokerPublisher,
    },
    {
      provide: MESSAGE_PUBLISHER,
      useClass: MessagePublisher,
    },
    {
      provide: TIMEZONE_FINDER,
      useClass: TimezoneFinder,
    },
  ],
  exports: [
    PrismaService,
    AdMapper,
    AD_REPOSITORY,
    PARAMS_PROVIDER,
    TIMEZONE_FINDER,
  ],
})
export class AdModule {}
