import { NotFoundException } from '@nestjs/common';
import { Test, TestingModule } from '@nestjs/testing';
import { FindAdByUuidQuery } from '../../../queries/find-ad-by-uuid.query';
import { AdsRepository } from '../../../adapters/secondaries/ads.repository';
import { FindAdByIdUseCase } from '../../../core/usecases/find-ad-by-uuid.usecase';
import { FindAdByIdRequestDTO } from '../../../interface/queries/find-ad-by-id/dtos/find-ad-by-id.request.dto';
import { MESSAGE_PUBLISHER } from '../../../../../app.constants';

const mockAd = {
  uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
};

const mockAdRepository = {
  findOneByUuid: jest
    .fn()
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    .mockImplementationOnce((query?: FindAdByUuidQuery) => {
      return Promise.resolve(mockAd);
    })
    .mockImplementation(() => {
      return Promise.resolve(null);
    }),
};

const mockMessagePublisher = {
  publish: jest.fn().mockImplementation(),
};

describe('FindAdByUuidUseCase', () => {
  let findAdByUuidUseCase: FindAdByIdUseCase;
  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [],
      providers: [
        {
          provide: AdsRepository,
          useValue: mockAdRepository,
        },
        {
          provide: MESSAGE_PUBLISHER,
          useValue: mockMessagePublisher,
        },
        FindAdByIdUseCase,
      ],
    }).compile();

    findAdByUuidUseCase = module.get<FindAdByIdUseCase>(FindAdByIdUseCase);
  });
  it('should be defined', () => {
    expect(findAdByUuidUseCase).toBeDefined();
  });
  describe('execute', () => {
    it('should return an ad', async () => {
      const findAdByUuidRequest: FindAdByIdRequestDTO =
        new FindAdByIdRequestDTO();
      findAdByUuidRequest.id = 'bb281075-1b98-4456-89d6-c643d3044a91';
      const ad = await findAdByUuidUseCase.execute(
        new FindAdByUuidQuery(findAdByUuidRequest),
      );
      expect(ad).toBe(mockAd);
    });
    it('should throw an error if ad does not exist', async () => {
      const findAdByUuidRequest: FindAdByIdRequestDTO =
        new FindAdByIdRequestDTO();
      findAdByUuidRequest.id = 'bb281075-1b98-4456-89d6-c643d3044a90';
      await expect(
        findAdByUuidUseCase.execute(new FindAdByUuidQuery(findAdByUuidRequest)),
      ).rejects.toBeInstanceOf(NotFoundException);
    });
  });
});
