import { Test, TestingModule } from '@nestjs/testing';
import { HealthCheckError, HealthIndicatorResult } from '@nestjs/terminus';
import { RepositoriesHealthIndicatorUseCase } from '../../core/usecases/repositories.health-indicator.usecase';
import { AD_REPOSITORY } from '@modules/health/health.di-tokens';
import { MESSAGE_PUBLISHER } from '@src/app.constants';
import { DatabaseErrorException } from '@libs/exceptions';

const mockAdRepository = {
  healthCheck: jest
    .fn()
    .mockImplementationOnce(() => {
      return Promise.resolve(true);
    })
    .mockImplementation(() => {
      throw new DatabaseErrorException('An error occured in the database');
    }),
};

const mockMessagePublisher = {
  publish: jest.fn().mockImplementation(),
};

describe('RepositoriesHealthIndicatorUseCase', () => {
  let repositoriesHealthIndicatorUseCase: RepositoriesHealthIndicatorUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        RepositoriesHealthIndicatorUseCase,
        {
          provide: AD_REPOSITORY,
          useValue: mockAdRepository,
        },
        {
          provide: MESSAGE_PUBLISHER,
          useValue: mockMessagePublisher,
        },
      ],
    }).compile();

    repositoriesHealthIndicatorUseCase =
      module.get<RepositoriesHealthIndicatorUseCase>(
        RepositoriesHealthIndicatorUseCase,
      );
  });

  it('should be defined', () => {
    expect(repositoriesHealthIndicatorUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should check health successfully', async () => {
      const healthIndicatorResult: HealthIndicatorResult =
        await repositoriesHealthIndicatorUseCase.isHealthy('repositories');
      expect(healthIndicatorResult['repositories'].status).toBe('up');
    });

    it('should throw an error if database is unavailable', async () => {
      jest.spyOn(mockMessagePublisher, 'publish');
      await expect(
        repositoriesHealthIndicatorUseCase.isHealthy('repositories'),
      ).rejects.toBeInstanceOf(HealthCheckError);
      expect(mockMessagePublisher.publish).toHaveBeenCalledTimes(1);
    });
  });
});
